@@ -1003,109 +1003,4 @@ else if (!is.null(signif)) names(lspline) <- paste(name,round(c(knots),signif),s
     return(lspline)
 }# }}}
 
-##' @export
-predictGLM <- function(object,newdata,id=NULL,fun=NULL,link.conf=TRUE,...) {# {{{
-    tt <- terms(object)
-    if (missing(newdata) || is.null(newdata)) {
-        mm <- X <- model.matrix(object)
-        mmDone <- TRUE
-        offset <- object$offset
-    }
-    else {
-        Terms <- delete.response(tt)
-        m <- model.frame(Terms, newdata, xlev = object$xlevels)
-        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
-    }
-    offsets <- rep(0,nrow(X))
-    if (!is.null(object$offset)) offsets <- m[, grep("offset", names(m))] 
-
-if (!is.null(object$family$linkinv)) linkinv <- object$family$linkinv  else linkinv <- function(x) x
-f <- function(p) { pp <- X %*% p+offsets; return(linkinv(pp)); }
-fl <- function(p) { pp <- X %*% p+offsets; return(pp); }
-if (!is.null(fun))  f <- fun
-
-if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)
-
-if (link.conf) { 
-     if (!is.null(id)) resl <- estimate(object,f=fl,id=id,...) else resl <- estimate(object,f=fl,...)
-     res <- linkinv(resl$coefmat[,c(1,3,4)]) 
-} else {
-if (!is.null(id)) res <- estimate(object,f=f,id=id,...) else res <- estimate(object,f=f,...)
-res <- res$coefmat
-}
-
-return(list(coef=coef,pred=res))
-}
-# }}}
-
-##' Reporting OR (exp(coef)) from glm with binomial link and glm predictions
-##' 
-##' Reporting OR from glm with binomial link  and glm predictions
-##' 
-##' @param object glm output 
-##' @param id possible id for cluster corrected standard errors
-##' @param fun possible function for non-standard predictions based on object
-##' @param ... arguments of estimate of lava for example level=0.95 
-##' @author Thomas Scheike
-##' @export
-##' @examples
-##' data(sTRACE)
-##' sTRACE$id <- sample(1:100,nrow(sTRACE),replace=TRUE)
-##'
-##' model <- glm(I(status==9)~sex+factor(diabetes)+age,data=sTRACE,family=binomial)
-##' summaryGLM(model)
-##' summaryGLM(model,id=sTRACE$id)
-##'
-##' nd <- data.frame(sex=c(0,1),age=67,diabetes=1)
-##' predictGLM(model,nd)
-##' @aliases predictGLM 
-summaryGLM <- function(object,id=NULL,fun=NULL,...) {# {{{
-
-f <- function(p) { pp <- exp(pp); return(pp); }
-
-if (!is.null(id)) coef <- estimate(object,id=id,...) else coef <- estimate(object,...)
-
-if (!is.null(id)) resl <- estimate(object,id=id,...) else resl <- estimate(object,...)
-res <- exp(resl$coefmat[,c(1,3,4)]) 
-
-if (!is.null(fun))  {
-if (!is.null(id)) resl <- estimate(object,id=id,f=fun,...) else resl <- estimate(object,f=fun,...)
-fout <- resl$coefmat
-} else fout <- NULL
-
-return(list(coef=coef,or=res,fout=NULL))
-}
-# }}}
-
-##' Wald test for model (type III test)
-##' 
-##' Wald test for model (type III test)
-##' 
-##' @param object, for example glm object that can be used with estimate
-##' @param ... arguments for estimate of lava for example id=data$id for cluster correction
-##' @author Thomas Scheike
-##' @export
-waldTest <- function(object,...)
-{# {{{
-Vars <-attr(terms(formula(object)),"term.labels")
-## remove possible cluster term
-clusterTerm<- grep("^cluster[(][A-z0-9._:]*",Vars,perl=TRUE)
-if (length(clusterTerm)==1) Vars <- Vars[-clusterTerm]
-Varf <- names(attr(attr(object$model,"terms"),"dataClasses"))[grep("factor",attr(attr(object$model,"terms"),"dataClasses"))]
-txtVar<-names(coef(object))
-###
-test.matrix <- matrix(0,length(Vars),3)
-colnames(test.matrix) <- c("Chisq","df","p.value")
-rownames(test.matrix) <- Vars
-k <- 1
-for (ff in Vars) {
-	pos <- grep(ff,txtVar,fixed=TRUE)
-        wt <- estimate(object,as.list(pos),...)$compare
-###	wt <- estimate(object,as.list(3:5))
-	test.matrix[k,] <- unlist(wt[1:3])
-	k <- k+1
-}
-return(test.matrix)
-}# }}}
-
 
