@@ -23,16 +23,17 @@
 ##' Events, deaths and censorings are specified via stop start structure and the Event call, that via a status vector 
 ##' and cause (code), censoring-codes (cens.code) and death-codes (death.code) indentifies these. See example and vignette. 
 ##'
-##' @param formula formula with 'EventCens' outcome
+##' @param formula formula with 'Event' outcome
 ##' @param data data frame
-##' @param cause of interest
-##' @param death.code codes for death (terminating event)
-##' @param cens.code code of censoring (1 default)
+##' @param cause of interest (1 default)
+##' @param death.code codes for death (terminating event, 2 default)
+##' @param cens.code code of censoring (0 default)
 ##' @param cens.model for stratified Cox model without covariates
 ##' @param weights weights for score equations
 ##' @param offset offsets for model
 ##' @param Gc censoring weights for time argument, default is to calculate these with a Kaplan-Meier estimator, should then give G_c(T_i-)
 ##' @param wcomp weights for composite outcome, so when cause=c(1,3), we might have wcomp=c(1,2).
+##' @param augmentation.type of augmentation when augmentation model is given 
 ##' @param ... Additional arguments to lower level funtions
 ##' @author Thomas Scheike
 ##' @examples
@@ -42,7 +43,7 @@
 ##' data(drcumhaz)
 ##' Lam1 <- base1cumhaz;  Lam2 <- base4cumhaz;  LamD <- drcumhaz
 ##' ## simulates recurrent events of types 1 and 2 and with terminal event D and censoring
-##' rr <- simRecurrentII(1000,Lam1,cumhaz2=Lam2,death.cumhaz=LamD,cens=3/5000)
+##' rr <- simRecurrentII(100,Lam1,cumhaz2=Lam2,death.cumhaz=LamD,cens=3/5000)
 ##' rr <- count.history(rr)
 ##' rr$cens <- 0
 ##' nid <- max(rr$id)
@@ -61,10 +62,22 @@
 ##'               death.code=3,cens.model=~strata(gx))
 ##' summary(lls)
 ##' 
-##' @aliases strataAugment scalecumhaz GLprediid recregIPCW twostageREC
+##' @aliases IIDbaseline.recreg strataAugment scalecumhaz GLprediid recregIPCW twostageREC simGLcox
 ##' @export
-recreg <- function(formula,data=data,cause=1,death.code=c(2),cens.code=0,cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,
-		   wcomp=NULL,...)
+recreg <- function(formula,data,cause=1,death.code=c(2),cens.code=0,cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,wcomp=NULL,
+		   augmentation.type=c("lindyn.augment","lin.augment"),...)
+{# {{{
+outi  <- recregB(formula,data,cause=cause,death.code=death.code,cens.code=cens.code,cens.model=cens.model,weights=weights,offset=offset,Gc=Gc,wcomp=wcomp,...)
+
+if (!is.null(outi$lindyn.augment)) {
+outA  <- recregB(formula,data,cause=cause,death.code=death.code,cens.code=cens.code,cens.model=cens.model,weights=weights,offset=offset,Gc=Gc,wcomp=wcomp,
+		 augmentation=outi[[augmentation.type[1]]],...)
+outi <- outA
+}
+return(outi)
+}# }}}
+
+recregB <- function(formula,data=data,cause=1,death.code=c(2),cens.code=0,cens.model=~1,weights=NULL,offset=NULL,Gc=NULL,wcomp=NULL,...)
 {# {{{
     cl <- match.call()# {{{
     m <- match.call(expand.dots = TRUE)[1:3]
@@ -119,16 +132,32 @@ recreg <- function(formula,data=data,cause=1,death.code=c(2),cens.code=0,cens.mo
 
     ## }}}
 
-    res <- c(recreg01(data,X,entry,exit,status,id=id,strata=strata,offset=offset,weights=weights,
-		      cens.model=cens.model, cause=cause, strata.name=strata.name, strataA=NULL,## strataAugment,
-		      death.code=death.code,cens.code=cens.code,Gc=Gc,wcomp=wcomp,...),
-             list(call=cl,model.frame=m,formula=formula,strata.pos=pos.strata,cluster.pos=pos.cluster,n=nrow(X),nevent=sum(status %in%cause))
-             )
+    res <- c(
+        recreg01(data, X, entry, exit, status,
+            id = id, strata = strata, offset = offset, weights = weights,
+            cens.model = cens.model, cause = cause, strata.name = strata.name, strataA = NULL, ## strataAugment,
+            death.code = death.code, cens.code = cens.code, Gc = Gc, wcomp = wcomp, ...
+        ),
+        list(call = cl, model.frame = m, formula = formula, strata.pos = pos.strata, cluster.pos = pos.cluster, n = nrow(X), nevent = sum(status %in% cause))
+    )
+    colnames(res$iid) <- names(res$coef)
 
-    class(res) <- c("phreg","recreg")
+    class(res) <- c("recreg", "phreg")
     return(res)
 }# }}}
 
+##' @export IIDbaseline.recreg 
+IIDbaseline.recreg <- function(x,time=NULL,fixbeta=NULL,...)
+{# {{{
+   return(IIDbaseline.cifreg(x,time=time,fixbeta=fixbeta,...))
+} # }}}
+
+
+##' @export
+IC.recreg <- function(x, ...) {
+  res <- with(x, iid * NROW(iid))
+  return(res)
+}
 recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,weights=NULL,strataA=NULL,
           strata.name=NULL,beta,stderr=1,method="NR",no.opt=FALSE, propodds=NULL,profile=0,
           case.weights=NULL,cause=1,death.code=2,cens.code=0,Gc=NULL,cens.model=~+1,augmentation=NULL,
@@ -189,6 +218,7 @@ recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,we
 	    case.weights <- case.weights* wwcomp
     }
     strata.call <- strata
+    call.id <- id
 
     if (!is.null(id)) {
         ids <- unique(id)
@@ -403,7 +433,7 @@ recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,we
         U <- U+augmentation
 
         out <- list(ploglik=ploglik,gradient=U,hessian=-DU,cox.prep=xx2,
-                    hessiantime=DUt,weightsJ=weightsJ,caseweightsJ=caseweightsJ,
+                    hessianttime=DUt,weightsJ=weightsJ,caseweightsJ=caseweightsJ,
                     jumptimes=jumptimes,strata=strataJ,nstrata=nstrata,S0s=cbind(S0oo,S0no),
                     time=jumptimes,S0=S0/(caseweightsJ*weightsJ),S2S0=S2S0,E=E,U=Ut,X=Xj,Gjumps=Gjumps)
 
@@ -529,7 +559,7 @@ recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,we
     varmc <-  crossprod(Uiid)
 
     ## compute regression augmentation for censoring martingale 
-    if ((!is.null(augment.model)) & (length(other)>1)  & (length(whereC)>0)) {## {{{
+    if ((!is.null(augment.model)) & (length(whereC)>0)) {## {{{
 
 	CovZXstrata <- function(X,Ej,Z,Sign,strata,nstrata,jumps) 
 	{# {{{
@@ -557,6 +587,7 @@ recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,we
 	###    Gcj <- exp(-cr2$cumhaz[,2])
 
     rr0 <- c(xx2$sign)
+    jumpsC <- which((xx2$Z[,1] %in% cens.code) & xx2$sign==1)
     strataCxx2 <- xx2$Z[,2]
     S0iC2  <-  S0iC <- rep(0,length(xx2$status))
     S0rrr <- revcumsumstrata(rr0,strataCxx2,nCstrata)
@@ -634,6 +665,7 @@ recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,we
    augment <- c(gamma %*% apply(ftime*UA/c(Gcj),2,sum))
    var.augment  <-  gamma %*% t(covXYdN) ###  /(nid^2)
 
+   if (!is.null(augmentation.call)) { ## update variance when called with augmentation
    #### iid magic  for censoring augmentation martingale{{{
    ### int_0^infty gamma (e_i - ebar(s)) 1/G_c(s) dM_i^c
    S0iG <- S0i <- rep(0,length(xx2$strata))
@@ -671,7 +703,10 @@ recreg01 <- function(data,X,entry,exit,status,id=NULL,strata=NULL,offset=NULL,we
    var.augment.times <-  varmc  +  iH %*% var.augment.times %*% iH
    var.augment.iid <-  crossprod(Uiid.augment) 
    var.augment.times.iid <- crossprod(Uiid.augment.times) 
-###   if (!is.null(augmentation)) varmc <- var.augment.times
+   } else {
+   var.augment <-  var.augment.times <-  var.augment.iid <-  var.augment.times.iid <- NULL
+   Uiid.augment.times <- Uiid.augment <- NULL
+   }
 
   } else {
     iid.augment <- iid.augment.times <- augment <- augment.times <- NULL 
@@ -732,15 +767,19 @@ if (no.opt==FALSE & p!=0) {
 DLambeta.t <- apply(opt$E/c(opt$S0),2,cumsumstrata,strata,nstrata)
 varbetat <-   rowSums((DLambeta.t %*% iH)*DLambeta.t)
 } else varbetat <- 0
-var.cumhaz <- cumsumstrata(1/opt$S0^2,strata,nstrata)+varbetat
+wwJ <- opt$caseweightsJ*opt$weightsJ
+var.cumhaz <- cumsumstrata(1/(opt$S0^2*wwJ),strata,nstrata)+varbetat
 se.cumhaz <- cbind(jumptimes,(var.cumhaz)^.5)
 colnames(se.cumhaz) <- c("time","se.cumhaz")
 
 out <- list(coef=beta.s,var=varmc,se.coef=diag(varmc)^.5,iid.naive=UUiid,
 	iid=Uiid,ncluster=nid,
-	ihessian=iH,hessian=opt$hessian,var1=var1,se1.coef=diag(var1)^.5,
+	ihessian=iH,hessian=opt$hessian,
+	hessianttime=opt$hessianttime,var1=var1,se1.coef=diag(var1)^.5,
 	ploglik=opt$ploglik,gradient=opt$gradient,
 	cumhaz=cumhaz, se.cumhaz=se.cumhaz,MGciid=MGc,
+	id=id.orig,call.id=call.id,
+	strata.jumps=opt$strata.jumps,
 	strata=xx2$strata,
 	nstrata=nstrata,strata.name=strata.name,strata.level=strata.level,
 	propodds=propodds,
@@ -765,9 +804,10 @@ if (cox.prep) out <- c(out,list(cox.prep=xx2))
 
 ##' @export
 recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
-	       cens.model=~1,km=TRUE,times=NULL,beta=NULL,offset=NULL,
+	       cens.model=~1,km=TRUE,times=NULL,beta=NULL,offset=NULL,type=c("incIPCW"),
 	       weights=NULL,model="exp",no.opt=FALSE,method="nr",augment.model=~+1,se=TRUE,...)
 {# {{{
+   ## type=c("incIPCW","IPCW","rate")
     cl <- match.call()# {{{
     m <- match.call(expand.dots = TRUE)[1:3]
     special <- c("strata", "cluster","offset")
@@ -824,6 +864,8 @@ recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
     orig.id <- id.orig <- id+1;
     nid <- length(unique(id))
 
+    ## to avoid Rcheck wawning
+
  ### setting up with artificial names
  data$status__ <-  status 
  data$id__ <-  id
@@ -833,11 +875,13 @@ recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
  data$death__ <- (status %in% death.code)*1
  data$entry__ <- entry 
  data$exit__ <- exit 
- data$statusC__ <- (status %in% cens.code)*1
+ statusC <- data$statusC__ <- (status %in% cens.code)*1
  data$status__cause <- (status %in% cause)*1
  data$rid__ <- revcumsumstrata(rep(1,length(entry)),id,nid)
  dexit <- exit
  dstatus <- status
+ ## to define properly 
+ Dtime <- NULL
 
   xr <- phreg(Surv(entry__,exit__,status__cause)~Count1__+death__+cluster(id__),data=data,no.opt=TRUE,no.var=1)
   formC <- update.formula(cens.model,Surv(entry__,exit__,statusC__)~ . +cluster(id__))
@@ -891,7 +935,17 @@ recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
   if (is.null(times)) stop("time for recurrent events regression must be given\n")
 
   ### setting up regression setting with Y(t) =\int_0^t 1/G(s) dN_i(s)
-  Ydata <- Y <- cumhazPiid <- sumstrata((xx$status!=0)*(xx$time<times)/Gc,xx$id,nid)
+ if (type[1]=="incIPCW") 
+  Ydata <- Y <- sumstrata((xx$status!=0)*(xx$time<times)/Gc,xx$id,nid)
+ else if (type[1]=="IPCW")  {
+     obs <- (exit<=time & (!statusC)) | (exit>=time)/cens.weights
+  Ydata <- Y <- sumstrata((xx$status!=0)*(xx$time<times),xx$id,nid)*obs
+  } else {
+     obs <- (exit<=time & (!statusC)) | (exit>=time)/cens.weights
+  NtD <- sumstrata((xx$status!=0)*(xx$time<times),xx$id,nid)
+  Ydata <- Y <- obs*NtD/pmin(times,Dtime)
+ }
+
   ## back to ordering in data
   Ydata <- Y <- Y[id[data$rid__==1]+1]
   ###
@@ -968,12 +1022,8 @@ recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
     if (length(val$coef) == length(colnames(X))) names(val$coef) <- colnames(X)
 
    val <- c(val, list(times = times, Y=Y, ncluster=nid, nevent=nevent, model.frame=m, n=length(exit),X=X))
-###	     formula = formula, formC = formC,
-###        exit = exit, cens.weights = cens.weights, cens.strata = cens.strata,
-###        cens.nstrata = cens.nstrata, model.frame = m, n = length(exit),
-###        nevent = nevent, ncluster = nid, Y = Y))
 
-    if (se) {# {{{
+    if (se & type[1]=="incIPCW") {# {{{
 
        Gcdata <- suppressWarnings(predict(cr,data,times=dexit,individual.time=TRUE,se=FALSE,km=km,tminus=TRUE)$surv)
        Gcdata[Gcdata<0.000001] <- 0.00001
@@ -999,11 +1049,39 @@ recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
        EdLam0 <- apply(E*c(S0i)*btime,2,cumsumstrata,xx$strata,xx$nstrata)
        MGt <- E[, drop = FALSE] - EdLam0 * c(xx$sign) 
        MGCiid <- apply(MGt, 2, sumstrata, xx$id, max(id) + 1)
-    }
-    else {
-        MGCiid <- 0
-    }# }}}
+    } else if (se & type[1]!="incIPCW") {
 
+    ### order of sorted times
+    ord <- resC$ord
+    X <-  X[ord,,drop=FALSE]
+    status <- status[ord]
+    exit <- exit[ord]
+    weights <- weights[ord]
+    offset <- offset[ord]
+    cens.weights <- cens.weights[ord]
+    lp <- c(X %*% val$coef+offset)
+    p <- expit(lp)
+    Y <- c((status %in% cause)*weights*(exit<=time)/cens.weights)
+
+    xx <- resC$cox.prep
+    S0i2 <- S0i <- rep(0,length(xx$strata))
+    S0i[xx$jumps+1]  <- 1/resC$S0
+    S0i2[xx$jumps+1] <- 1/resC$S0^2
+    ## compute function h(s) = \sum_i X_i Y_i(t) I(s \leq T_i \leq t) 
+    ## to make \int h(s)/Ys  dM_i^C(s) 
+    h  <-  apply(X*Y,2,revcumsumstrata,xx$strata,xx$nstrata)
+    ### Cens-Martingale as a function of time and for all subjects to handle strata 
+    ## to make \int h(s)/Ys  dM_i^C(s)  = \int h(s)/Ys  dN_i^C(s) - dLambda_i^C(s)
+    IhdLam0 <- apply(h*S0i2,2,cumsumstrata,xx$strata,xx$nstrata)
+    U <- matrix(0,nrow(xx$X),ncol(X))
+    U[xx$jumps+1,] <- h[xx$jumps+1,] /c(resC$S0)
+    MGt <- (U[,drop=FALSE]-IhdLam0)*c(xx$weights)
+
+    ### Censoring Variance Adjustment  \int h^2(s) / y.(s) d Lam_c(s) estimated by \int h^2(s) / y.(s)^2  d N.^C(s) 
+    MGCiid <- apply(MGt,2,sumstrata,xx$id,max(id)+1)
+   }  else { 
+	  MGCiid <- 0
+  }## }}}
 
     if (se) val$MGciid <- MGCiid %*% val$ihessian else val$MGciid <- MGCiid 
     val$id <- id
@@ -1030,12 +1108,21 @@ recregIPCW <- function(formula,data=data,cause=1,cens.code=0,death.code=2,
 ##' @export
 strataAugment <- survival:::strata
 
+##' @export
 simGLcox <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rc=NULL,fz=NULL,fdz=NULL,
-		     model=c("twostage","frailty","shared","multiplicative"),type=NULL,share=1,cens=NULL,nmax=200)
+   model=c("twostage","frailty","shared","multiplicative"),type=NULL,share=1,cens=NULL,nmin=100,nmax=1000)
 {# {{{
 ## setting up baselines for simulations 
-base1 <- predictCumhaz(rbind(0,as.matrix(base1)),1:round(tail(base1[,1],1)) )
-cumD <- predictCumhaz(rbind(0,as.matrix(drcumhaz)),base1[,1])
+maxt <- tail(base1[,1],1)
+base1 <- as.matrix(base1); drcumhaz <- as.matrix(drcumhaz)
+nmin <- max(nrow(base1),nrow(drcumhaz),nmin)
+nmin <- min(nmax,nmin)
+seqt <- seq(from=0,to=maxt,length.out=nmin)
+if (base1[1,1]!=0) base1 <- rbind(0,base1) 
+if (drcumhaz[1,1]!=0) drcumhaz <- rbind(0,drcumhaz) 
+base1 <- cbind(seqt, lin.approx(seqt,base1))
+cumD <- cbind(seqt, lin.approx(seqt,drcumhaz))
+###
 St <- exp(-cumD[,2])
 Stm <- cbind(base1[,1],St)
 ###
@@ -1074,7 +1161,9 @@ if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}
 
  ## survival censoring given X, Z, either twostage or frailty-model 
  if (type>=2) stype <- 2 else stype <- 1
- dd <- .Call("_mets_simSurvZ",as.matrix(rbind(c(0,1),Stm)),rd,z,var.z[1],stype)
+ if (var.z[1]==0) stype <- 1
+### dd <- .Call("_mets_simSurvZ",as.matrix(rbind(c(0,1),Stm)),rd,z,var.z[1],stype)
+ dd <- .Call("_mets_simSurvZ",as.matrix(rbind(Stm)),rd,z,var.z[1],stype)
  dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))
  if (!is.null(cens)) cens <- rexp(n)/(rc*cens) else cens <- rep(maxtime,n)
  dd$status <- ifelse(dd$time<cens,dd$status,0)
@@ -1090,7 +1179,90 @@ if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}
      type <- 3
  }
  ## type=2 draw recurrent process given X,Z with rate:
- ##  1/S(t|X,Z) exp(X^t beta_1) d \Lambda_1(t)
+ ##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
+ ## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
+ ## type=3, observed hazards on Cox form among survivors
+ ## W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
+ ## or W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
+ ## or W_2 * W_1 ~ N1, W_1~ D   observed hazards on Cox form among survivors
+ dcum <- cbind(base1[,1],dbase1)
+### ll <- .Call("_mets_simGL",as.matrix(rbind(0,dcum)),c(1,St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
+ ll <- .Call("_mets_simGL",as.matrix(dcum),c(St),r1,rd,z1,fzz,dd$time,type,var.z[1],nmax,1)
+ colnames(ll) <- c("id","start","stop","death")
+ ll <- data.frame(ll)
+ ll$death <- dd$status[ll$id+1]
+ ## add frailty to data for possible validation
+ ll$z <- z1[ll$id+1]
+ ll$fz <- fzz[ll$id+1]
+ ## add counts of id
+ ids <- countID(ll)
+ ll <- cbind(ll,ids[,c(2,4,5)]); 
+ ll$status <- 1; 
+ ll <- dtransform(ll,status=0,reverseCountid==1)
+ ll$statusD <- ll$status
+ ll <- dtransform(ll,statusD=3,reverseCountid==1 & death==1)
+
+ return(ll)
+}# }}}
+
+simGLcoxC <- function(n,base1,drcumhaz,var.z=0,r1=NULL,rd=NULL,rc=NULL,fz=NULL,fdz=NULL,
+	     model=c("twostage","frailty","shared","multiplicative"),type=NULL,share=1,cens=NULL,nmax=200,by=1)
+{# {{{
+## setting up baselines for simulations 
+maxt <- tail(base1[,1],1)
+seqt <- seq(by,maxt,by=by)
+base1 <- predictCumhaz(rbind(0, as.matrix(base1)), seqt)
+cumD <- predictCumhaz(rbind(0, as.matrix(drcumhaz)), seqt)
+###
+St <- 1-cumD[,2]
+Stm <- cbind(base1[,1],St)
+###
+dbase1 <- diff(c(0,base1[,2]))
+dcum <- cbind(base1[,1],dbase1)
+maxtime <- tail(base1[,1],1)
+
+if (is.null(r1)) r1 <- rep(1,n)
+if (is.null(rd)) rd <- rep(1,n)
+if (is.null(rc)) rc <- rep(1,n)
+
+fz.orig <- fz
+if (is.null(fz)) fz <- function(x) x
+
+ if (var.z[1]>0) {
+	 z1 <- z <- rgamma(n,share/var.z[1])*var.z[1] 
+	 if (share<1) { 
+		 z2 <- rgamma(n,(1-share)/var.z[1])*var.z[1] 
+		 z <- z+z2
+	 } 
+	 fzz <- fz(z1)
+	 if (share<1) fzz <- fzz/share; 
+	 mza <- mean(fzz)
+	 if (n<10000 & (!is.null(fz.orig))) {
+	    zl <- rgamma(100000,share/var.z[1])*var.z[1] 
+	    fzl <- fz(z)
+	    mza <- mean(fzl)
+	 } 
+ }  else fzz <- z <- z1 <- rep(1,n)
+
+if (var.z[1]==0) model <- "frailty"
+if (is.null(type)) 
+if (model[1]=="twostage") type <- 2 else type <- 1
+## for frailty setting we also consider any function of z 
+if (!is.null(fdz)) { fdzz <- fdz(z); rd <- rd*fdzz; z <- rep(1,n);}
+
+ ## survival censoring given X, Z, either twostage or frailty-model 
+ if (type>=2) stype <- 2 else stype <- 1
+ cumHazS <- -log(St)
+ dd <- rchaz(cumHazS,rd)
+ dd <- data.frame(time=dd[,1],status=(dd[,1]<maxtime))
+ if (!is.null(cens)) cens <- rexp(n)/(rc*cens) else cens <- rep(maxtime,n)
+ dd$status <- ifelse(dd$time<cens,dd$status,0)
+ dd$time <- pmin(dd$time,cens)
+
+ ## to avoid R check error
+ reverseCountid  <-  death  <- NULL
+ ## type=2 draw recurrent process given X,Z with rate:
+ ##  Z exp(X^t beta_1) d \Lambda_1(t)/S(t|X,Z) 
  ## such that GL model holds with exp(X^t beta_1) \Lambda_1(t)
  ## type=3, observed hazards on Cox form among survivors
  ## W_1 ~ N1, W_1+W_2 ~ D observed hazards on Cox form among survivors
@@ -1253,34 +1425,45 @@ out <- FGprediid(...,model="GL")
 return(out)
 }# }}}
 
-boottwostageREC <- function(margsurv,recurrent,data,bootstrap=100,id="id",...) 
+boottwostageREC <- function(margsurv,recurrent,data,bootstrap=100,id="id",stepsize=0.5,...) 
 {# {{{
 n <- max(margsurv$id)
 K <- bootstrap
-  formid <- as.formula(paste("~",id))
-  rrb <- blocksample(data, size = n*K, formid)
-  rrb$strata <- floor((rrb[,id]-0.01)/n)
+formid <- as.formula(paste("~",id))
+rrb <- blocksample(data, size = n*K, formid)
+rrb$strata <- floor((rrb[,id]-0.01)/n)
 
-  outb <- c()
+  outb <- outd <- outr <- c()
   for (i in 1:K)
   {
      rrbs <- subset(rrb,strata==i-1)
      drb <- phreg(margsurv$formula,data=rrbs)
-     xrb <- phreg(recurrent$formula,data=rrbs)
-     outbl <- tryCatch(twostageREC(drb,xrb,rrbs,...),error=function(x) NULL)
+    if (inherits(recurrent,"recreg")) {
+	 xrb <- recreg(recurrent$formula,data=rrbs,
+      cause=recurrent$cause,death.code=recurrent$death.code,cens.code=recurrent$cens.code,cox.prep=TRUE)
+    } else xrb <- phreg(recurrent$formula,data=rrbs)
+     outbl <- tryCatch(twostageREC(drb,xrb,rrbs,numderiv=0,control=list(stepsize=stepsize),...),error=function(x) NULL)
      if (!is.null(outbl)) outb <- rbind(outb,outbl$coef)
+     outd <- rbind(outd,coef(drb))
+     outr <- rbind(outr,coef(xrb))
   }
-  var <- var(outb)
+  varb <- cov(outb)
+  vard <- cov(outd)
+  varr <- cov(outr)
 
-  list(var=var, se=diag(var)^.5,outb=outb)
+  list(outb=outb,var=varb,se=diag(varb)^.5,se.coxD=diag(vard)^.5,se.coxR=diag(varr)^.5)
 }# }}}
 
 ##' @export
-twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NULL, model=c("full","shared","non-shared"),
-  theta.des = NULL, var.link = 0, method = "NR", no.opt = FALSE, weights = NULL, se.cluster = NULL, nufix=0,nu=NULL,at.risk=1,...)
+twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NULL, model=c("full","shared","non-shared"),ghosh.lin=NULL,
+  theta.des = NULL, var.link = 0, method = "NR", no.opt = FALSE, weights = NULL, se.cluster = NULL, 
+  fnu=NULL,nufix=0,nu=NULL,at.risk=1,numderiv=1,derivmethod=c("simple","Richardson"),...)
 {# {{{
     if (!inherits(margsurv, "phreg")) stop("Must use phreg for death model\n")
     if (!inherits(recurrent, "phreg")) stop("Must use phreg for recurrent model\n")
+    if (is.null(recurrent$cox.prep)) stop("recreg must be called with cox.prep=TRUE\n")
+    if (inherits(recurrent, "recreg") & is.null(ghosh.lin))  ghosh.lin <- 1 
+    if ((!inherits(recurrent, "recreg")) & is.null(ghosh.lin))  ghosh.lin <- 0 
     clusters <- margsurv$cox.prep$id
     n <-  max(clusters)+1
     if (is.null(theta.des) == TRUE) ptheta <- 1
@@ -1291,7 +1474,7 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
         if (var.link == 1) theta <- rep(0, ptheta)
         if (var.link == 0) theta <- rep(1, ptheta)
     }
-    if (is.null(nu)  & (nufix==0)) nu <- 0 
+    if (is.null(nu)  & (nufix==0)) { if (is.null(fnu))  nu <-  0.5 else nu <- fnu[[1]](0.5) }
     if (length(theta) != ptheta) theta <- rep(theta[1], ptheta)
     if (length(nu) != ptheta) nu <- rep(nu,ptheta)
     theta.score <- rep(0, ptheta)
@@ -1358,7 +1541,7 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
 ###        tildeL <- .Call("_mets_tildeLambda1",S01i,cumhazD,r1,rd,thetai,xx$id)
 ###	if (at.risk==1)  
 ###		tildeL <- apply(tildeL*c(xr$sign),2,cumsumstrata,xr$id,mid)
-        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai,xr$id,xr$sign)
+        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai+1*ghosh.lin,xr$id,xr$sign)
         tildeLast <- tildeL[lastid,]
 	Ht <- thetav*tildeL[,1]+exp(thetav*HD)
 	Hr <- thetai*tildeLast[,1]+exp(thetai*cumDL)
@@ -1464,7 +1647,9 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
     }
 # }}}
 
-   fw <- function(x) 1/(1+exp(x))
+    ## default is simple identity 
+###    if (is.null(fnu)) { fw <- function(x) 1/(1+exp(x)); Dfw <- function(x) -exp(x)/(1+exp(x))^2;} else { 
+   if (is.null(fnu)) { fw <- function(x) x; Dfw <- function(x) 1;} else { fw <- fnu[[1]]; Dfw <- fnu[[2]]; } 
    nudes <- theta.des  
    p <- ncol(theta.des)
 
@@ -1478,10 +1663,7 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
         thetai <- thetav[firstid]; tbeta1i <- tbeta1[firstid]; tbeta2i <- tbeta2[firstid]
 	###
 	R <-  exp( - thetav*HD);  DR <- -HD*exp( - thetav* HD); D2R <-  HD^2*exp( - thetav* HD) 
-###        tildeL <- .Call("_mets_tildeLambda1",S01i,cumhazD,r1,rd,thetai,xx$id)
-###	if (at.risk==1)  
-###		tildeL <- apply(tildeL*c(xr$sign),2,cumsumstrata,xr$id,mid)
-        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai,xr$id,xr$sign)
+        tildeL <- .Call("_mets_tildeLambda1R",S01i,cumhazD,r1,rd,thetai+1*ghosh.lin,xr$id,xr$sign)
 	tildeLast <- tildeL[lastid,]
 	Ht <- (thetav/tbeta1)*tildeL[,1]+exp(thetav*HD)
 	Hr <- (thetai/tbeta1i)*tildeLast[,1]+exp(thetai*cumDL)
@@ -1532,7 +1714,7 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
         ###
 	if (nufix==1)
         scoreiid <- thetaX * c(Dltheta)
-        else  scoreiid <- cbind(thetaX * c(Dltheta),nuX*c(Dlnu)*(-exp(nu1i)*tbeta1i^2))
+        else  scoreiid <- cbind(thetaX * c(Dltheta),nuX*c(Dlnu)*Dfw(nu1i))
         ###   ###
         Dl11s <- -sumstrata(N1sum$lagsum^2/(tbeta1 + thetav * N1sum$lagsum)^2 * statusxb, xx$id, mid)
         Dl3s <- (2*tbeta1i/thetai^2) * DHr/Hr -(tbeta1i/thetai+ N1i.tau)*(D2Hr*Hr-DHr^2)/Hr^2 - (2*tbeta1i/thetai^3) * log(Hr) 
@@ -1554,9 +1736,7 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
 ###	    hessian <- rbind(hessian,cbind(t(hessianp),hessiann))
 	}
         hess2 <- crossprod(scoreiid)
-        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid,
-            ploglik = ploglik, gradient = gradient, hessian = -hess2,
-            hess2 = hess2)
+        val <- list(id = xx$id, score.iid = scoreiid, logl.iid = logliid, ploglik = ploglik, gradient = gradient, hessian = -hess2, hess2 = hess2)
         if (all)
             return(val)
 ###        with(val, structure(-ploglik, gradient = -gradient, hessian = -hessian))
@@ -1595,6 +1775,16 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
         rownames(theta) <- thetanames
         names(val$coef)  <- thetanames
     }
+
+    if (numderiv==1 & model[1]=="shared") {
+	    dobj <- function(p) {
+		    oo <- obj(p)
+		    return(attr(oo,"gradient"))
+	    }
+	    hessian <- numDeriv::jacobian(dobj,val$coef,method=derivmethod[1])
+	    val$hessian <- -hessian
+    }
+
     hessianI <- solve(val$hessian)
     val$theta.iid.naive <- val$score.iid %*% hessianI
 
@@ -1615,9 +1805,11 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
     val <- c(val, list(theta = theta, var.theta = var,n=mid,p=ncol(thetaX),var.link=var.link,
                        robvar.theta = var, var = var, thetanames = thetanames,
                        model = model[1], se = diag(var)^0.5), 
-	     var.naive = naive.var,no.opt=no.opt)
+	     var.naive = naive.var,no.opt=no.opt,ghosh.lin=ghosh.lin)
     class(val) <- "twostageREC"
     attr(val, "clusters") <- clusters
+    attr(val, "fnu") <- fw
+    attr(val, "Dfnu") <- Dfw
     attr(val, "secluster") <- c(se.cluster)
     attr(val, "var.link") <- var.link
     attr(val, "ptheta") <- ptheta
@@ -1630,30 +1822,30 @@ twostageREC  <-  function (margsurv,recurrent, data = parent.frame(), theta = NU
 # }}}
 
 ##' @export
-summary.twostageREC <- function(object,...) {# {{{
+summary.twostageREC <- function(object,vcov=NULL,delta=0,...) {# {{{
     I <- -solve(object$hessian)
-    V <- object$var
+    if (!is.null(vcov)) V <- vcov else V <- object$var
     ncluster <- object$n
-    cc <- cbind(object$coef,diag(V)^0.5,diag(I)^0.5)
-    cc  <- cbind(cc,2*(pnorm(abs(cc[,1]/cc[,2]),lower.tail=FALSE)))
-    colnames(cc) <- c("Estimate","S.E.","dU^-1/2","P-value")
-    if (length(class(object))==1) if (!is.null(ncluster <- attributes(V)$ncluster))
-    rownames(cc) <- names(coef(object))
+    cc <- estimate(coef=object$coef,vcov=V)$coefmat
     pd <- object$p
     if (object$var.link==1 & object$model=="full") f <- function(p) exp(p)
     if (object$var.link==0 & object$model=="full") f <- function(p) p
-    if (object$var.link==1 & object$model=="shared") f <- function(p) c(exp(p[1:pd]),1/(1+exp(p[(pd+1):2*pd])))
-    if (object$var.link==0 & object$model=="shared") f <- function(p) c(p[1:pd],1/(1+exp(p[(pd+1):2*pd])))
-    expC <- lava::estimate(coef=object$coef,vcov=V,f=f)$coefmat[,c(1,3,4),drop=FALSE]
+    if (object$var.link==1 & object$model=="shared") f <- function(p) c(exp(p[1]),attr(object,"fnu")(p[2]))
+    if (object$var.link==0 & object$model=="shared") f <- function(p) c(p[1],attr(object,"fnu")(p[2]))
+    if (delta==1 | object$model=="full") 
+    expC <- lava::estimate(coef=object$coef,vcov=V,f=f)$coefmat ##[,c(1,3,4),drop=FALSE]
+    else expC <- apply(cc[,c(1,3,4),drop=FALSE],2,f) 
   n <- object$n
-  res <- list(coef=cc,n=n,nevent=object$nevent,ncluster=ncluster,var=V,exp.coef=expC,var.link=object$var.link)
+  res <- list(coef=cc,n=n,nevent=object$nevent,ncluster=ncluster,var=V,exp.coef=expC,var.link=object$var.link,
+	      ghosh.lin=object$ghosh.lin)
   class(res) <- "summary.twostageREC"
   res
-}
-# }}}
+} # }}}
 
 ##' @export
 print.summary.twostageREC  <- function(x,max.strata=5,...) {# {{{
+  if (x$ghosh.lin==0) cat("Cox(recurrent)-Cox(terminal) intensity model\n"); 
+  if (x$ghosh.lin==1) cat("Ghosh-Lin(recurrent)-Cox(terminal) mean model\n"); 
   if (!is.null(x$ncluster)) cat("\n ", x$ncluster, " clusters\n",sep="")
   if (!is.null(x$coef)) {
     cat("coeffients:\n")
@@ -1666,3 +1858,184 @@ print.summary.twostageREC  <- function(x,max.strata=5,...) {# {{{
   cat("\n")
 } # }}}
 
+##' @export
+print.twostageREC  <- function(x,...) {# {{{
+  print(summary(x),...)
+} # }}}
+
+##' Evaluates piece constant covariates at min(D,t) where D is a terminal event
+##'
+##' returns X(min(D,t)) and min(D,t) and their ratio. for censored observation 0. 
+##' to use with the IPCW models implemented. 
+##'
+##' @param formula formula with 'Event' outcome and X to evaluate at min(D,t)
+##' @param data data frame
+##' @param death.code codes for death (terminating event, 2 default)
+##' @param time for evaluation 
+##' @author Thomas Scheike
+##' @export
+evalTerminal <- function(formula,data=data,death.code=2,time=NULL)
+{# {{{
+    cl <- match.call()# {{{
+    m <- match.call(expand.dots = TRUE)[1:3]
+    special <- c("strata", "cluster","offset")
+    Terms <- terms(formula, special, data = data)
+    m$formula <- Terms
+    m[[1]] <- as.name("model.frame")
+    m <- eval(m, parent.frame())
+    Y <- model.extract(m, "response")
+    if (!inherits(Y,"Event")) stop("Expected a 'Event'-object")
+    if (ncol(Y)==2) {
+        exit <- Y[,1]
+        entry <- NULL ## rep(0,nrow(Y))
+        status <- Y[,2]
+    } else {
+        entry <- Y[,1]
+        exit <- Y[,2]
+        status <- Y[,3]
+    }
+    id <- strata <- NULL
+    if (!is.null(attributes(Terms)$specials$cluster)) {
+        ts <- survival::untangle.specials(Terms, "cluster")
+        pos.cluster <- ts$terms
+        Terms  <- Terms[-ts$terms]
+        id <- m[[ts$vars]]
+    } else pos.cluster <- NULL
+###    if (!is.null(stratapos <- attributes(Terms)$specials$strata)) {
+###        ts <- survival::untangle.specials(Terms, "strata")
+###        pos.strata <- ts$terms
+###        Terms  <- Terms[-ts$terms]
+###        strata <- m[[ts$vars]]
+###        strata.name <- ts$vars
+###    }  else { strata.name <- NULL; pos.strata <- NULL}
+###    if (!is.null(offsetpos <- attributes(Terms)$specials$offset)) {
+###        ts <- survival::untangle.specials(Terms, "offset")
+###        Terms  <- Terms[-ts$terms]
+###        offset <- m[[ts$vars]]
+###    }
+    X <- model.matrix(Terms, m)
+    if (!is.null(intpos  <- attributes(Terms)$intercept))
+        X <- X[,-intpos,drop=FALSE]
+    if (ncol(X)==0) X <- matrix(nrow=0,ncol=0)
+    ## }}}
+
+   if (!is.null(id)) {
+	   call.id <- id
+        ids <- unique(id)
+        nid <- length(ids)
+        if (is.numeric(id))
+            id <-  fast.approx(ids,id)-1
+        else  {
+            id <- as.integer(factor(id,labels=seq(nid)))-1
+        }
+    } else { call.id <- id <- as.integer(seq_along(entry))-1;  nid <- nrow(X); }
+    ## orginal id coding into integers 1:...
+   id <- id+1
+
+   dd <- data.frame(id=id)
+   dd <- countID(dd,sorted=TRUE)
+   ## new id 1,2,.... and so on, referring to rows of data
+   id <- dd$indexid+1
+
+ ###
+ indexD  <- which(exit <= time & (status %in% death.code))
+ ### rr[indexD,c("time1","statusD")]
+ idD <- id[indexD]
+ Dmintid <- rep(0,nid)
+ Dmintid[idD] <- exit[indexD]
+ ### 
+ indexA<- which(entry <= time & time <= exit)
+ idA <- id[indexA]
+ Dmintid[idA] <- time
+ Dmint <- Dmintid[id]
+ ###
+ XminDtid <- matrix(0,nid,ncol(X))
+ colnames(XminDtid) <- colnames(X)
+ XminDtid[idD,] <- X[indexD,]
+ XminDtid[idA,]  <- X[indexA,]
+ XminDt <- XminDtid[id,,drop=FALSE]
+ ratio <- XminDt/Dmint
+ ratio[is.na(ratio)] <- 0
+
+ dd <- data.frame(cbind(XminDt,Dmint,id,call.id,ratio))
+ colnames(dd) <- c("XminDt","minDt","nid","call.id","ratio")
+
+ return(dd)
+} # }}}
+
+dynCensAugOld <- function(formC,data,augmentC=~+1,response="Yipcw",time=NULL,Z=NULL) {# {{{ 
+if (is.null(time)) stop("must give time of response \n")
+   data$Y__ <- data[,response]
+   varsC <- c("Y__",attr(terms(augmentC), "term.labels"))
+   formCC <- update(formC, reformulate(c(".", varsC)))
+###	www <- rep(1,nrow(data))
+###	if (treat.specific.cens==1)  www <-data$WW1__;  
+   cr2 <- phreg(formCC, data = data, no.opt = TRUE, no.var = 1,Z=Z)
+   xx <- cr2$cox.prep
+   icoxS0 <- rep(0,length(cr2$S0))
+   icoxS0[cr2$S0>0] <- 1/cr2$S0[cr2$S0>0]
+   S0i <- rep(0, length(xx$strata))
+   S0i[xx$jumps + 1] <- icoxS0
+   km <- TRUE
+   if (!km) {
+        cumhazD <- c(cumsumstratasum(S0i, xx$strata, xx$nstrata)$lagsum)
+        St <- exp(-cumhazD)
+   } else St <- c(exp(cumsumstratasum(log(1 - S0i), xx$strata, xx$nstrata)$lagsum))
+
+   nterms <- cr2$p-1
+   dhessian <- cr2$hessianttime
+   dhessian <-  .Call("XXMatFULL",dhessian,cr2$p,PACKAGE="mets")$XXf
+   ###  matrix(apply(dhessian,2,sum),3,3)
+   timeb <- which(cr2$cumhaz[,1]<time)
+   ### take relevant \sum H_i(s,t) (e_i - \bar e)
+   covts <- dhessian[timeb,1+1:nterms,drop=FALSE]
+   ### construct relevant \sum (e_i - \bar e)^2
+   Pt <- dhessian[timeb,-c((1:(nterms+1)),(1:(nterms))*(nterms+1)+1),drop=FALSE]
+   ###  matrix(apply(dhessian[,c(5,6,8,9)],2,sum),2,2)
+   gammatt <- .Call("CubeVec",Pt,covts,1,PACKAGE="mets")$XXbeta
+   S0 <- cr2$S0[timeb]
+   gammatt[is.na(gammatt)] <- 0
+   gammatt[gammatt==Inf] <- 0
+   Gctb <- St[cr2$cox.prep$jumps+1][timeb]
+   augmentt.times <- apply(gammatt*cr2$U[timeb,1+1:nterms,drop=FALSE],1,sum)
+   augment.times <- sum(augmentt.times)
+   if (!is.null(Z)) {
+             Zj <- cr2$cox.prep$Z[cr2$cox.prep$jumps+1,][timeb]
+             Xaugment.times <- apply( augmentt.times*Zj,2,sum)
+   }
+
+   p <- 1
+   #### iid magic  for censoring augmentation martingale ## {{{
+   ### int_0^infty gamma(t) (e_i - ebar(s)) 1/G_c(s) dM_i^c
+   xx <- cr2$cox.prep
+   nid <- max(xx$id)+1
+   jumpsC <- xx$jumps+1
+   rr0 <- xx$sign
+   S0i <- rep(0,length(xx$strata))
+   S0i[jumpsC] <- c(1/(icoxS0*St[jumpsC]))
+   S0i[jumpsC] <- icoxS0
+
+   pXXA <- ncol(cr2$E)-1
+   EA <- cr2$E[timeb,-1,drop=FALSE]
+   gammasEs <- .Call("CubeMattime",gammatt,EA,pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
+   gammasE <- matrix(0,length(xx$strata),1)
+   gammattt  <-    matrix(0,length(xx$strata),pXXA*1)
+   jumpsCt <- jumpsC[timeb]
+   gammasE[jumpsCt,] <- gammasEs
+   gammattt[jumpsCt,] <- gammatt
+   gammaEsdLam0 <- apply(gammasE*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
+   gammadLam0 <-   apply(gammattt*S0i,2,cumsumstrata,xx$strata,xx$nstrata)
+   XgammadLam0 <- .Call("CubeMattime",gammadLam0,xx$X[,-1],pXXA,p,pXXA,1,0,1,0,PACKAGE="mets")$XXX
+   Ut <- Et <- matrix(0,length(xx$strata),1)
+   Ut[jumpsCt,] <- augmentt.times
+   MGCt <- Ut[,drop=FALSE]-(XgammadLam0-gammaEsdLam0)*c(rr0)
+   MGCiid <- apply(MGCt,2,sumstrata,xx$id,nid)
+   MGCiid <- MGCiid/nid
+   ## }}}
+
+   nid <- max(cr2$id)
+   ids <- headstrata(cr2$id-1,nid)
+   ids <- cr2$call.id[ids]
+
+   res <- list(MGCiid=MGCiid,gammat=gammatt,augment=augment.times, id=ids,n=nid)
+} # }}}
