@@ -1,7 +1,11 @@
-## ---- include = FALSE---------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = TRUE,
-  #dev="png",
+  ##dev="png",
+  dpi=50,
+  fig.width=7.15, fig.height=5.5,
+  out.width="600px",
+  fig.retina=1,
   comment = "#>"  
 )
 library(mets)
@@ -23,10 +27,10 @@ library(mets)
 
 ## -----------------------------------------------------------------------------
    # making iid decomposition of regression parameters
-   betaiid <- iid(out)
+   betaiid <- IC(out)
    head(betaiid)
    # robust standard errors
-   crossprod(betaiid)^.5
+   crossprod(betaiid/NROW(betaiid))^.5
    # same as 
 
 ## -----------------------------------------------------------------------------
