@@ -61,27 +61,27 @@ library(mets)
  plot(margph)
 
 ## ----label=survival-pairwise1, eval=fullVignette------------------------------
-#  # Clayton-Oakes, MLE , overall variance
-#  fitco1<-twostageMLE(margph,data=prt,theta=2.7)
+# # Clayton-Oakes, MLE , overall variance
+# fitco1<-twostageMLE(margph,data=prt,theta=2.7)
 
 ## -----------------------------------------------------------------------------
  summary(fitco1)
 
 ## ----label=survival-pairwise2, eval=fullVignette------------------------------
-#  fitco2 <- survival.twostage(margph,data=prt,theta=2.7,clusters=prt$id,var.link=0)
+# fitco2 <- survival.twostage(margph,data=prt,theta=2.7,clusters=prt$id,var.link=0)
 
 ## -----------------------------------------------------------------------------
  summary(fitco2)
 
 ## ----label=survival-pairwise3, eval=fullVignette------------------------------
-#   mm <- model.matrix(~-1+factor(zyg),prt)
-#   fitco3<-twostageMLE(margph,data=prt,theta=1,theta.des=mm)
+#  mm <- model.matrix(~-1+factor(zyg),prt)
+#  fitco3<-twostageMLE(margph,data=prt,theta=1,theta.des=mm)
 
 ## -----------------------------------------------------------------------------
  summary(fitco3)
 
 ## ----label=survival-pairwise4, eval=fullVignette------------------------------
-#  fitco4 <- survival.twostage(margph,data=prt,theta=1,clusters=prt$id,var.link=0,theta.des=mm)
+# fitco4 <- survival.twostage(margph,data=prt,theta=1,clusters=prt$id,var.link=0,theta.des=mm)
 
 ## -----------------------------------------------------------------------------
  summary(fitco4)
@@ -93,36 +93,36 @@ library(mets)
  kendall.ClaytonOakes.twin.ace(fitco4$theta[1],0,K=1000)$mz.kendall
 
 ## ----label=survival-polygenic1, eval=fullVignette-----------------------------
-#   ### setting up design for random effects and parameters of random effects
-#   desace <- twin.polygen.design(prt,type="ace")
-#  
-#   ### ace model
-#   fitace <- survival.twostage(margph,data=prt,theta=1,
-#         clusters=prt$id,var.link=0,model="clayton.oakes",
-#         numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
+#  ### setting up design for random effects and parameters of random effects
+#  desace <- twin.polygen.design(prt,type="ace")
+# 
+#  ### ace model
+#  fitace <- survival.twostage(margph,data=prt,theta=1,
+#        clusters=prt$id,var.link=0,model="clayton.oakes",
+#        numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
 
 ## -----------------------------------------------------------------------------
  summary(fitace)
 
 ## ----label=survival-polygenic2, eval=fullVignette-----------------------------
-#   ### ace model with positive random effects variances
-#   # fitacee <- survival.twostage(margph,data=prt,theta=1,
-#   #      clusters=prt$id,var.link=1,model="clayton.oakes",
-#   #      numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
-#   #summary(fitacee)
-#  
-#   ### ae model
-#   #desae <- twin.polygen.design(prt,type="ae")
-#   #fitae <- survival.twostage(margph,data=prt,theta=1,
-#   #      clusters=prt$id,var.link=0,model="clayton.oakes",
-#   #      numDeriv=1,random.design=desae$des.rv,theta.des=desae$pardes)
-#   #summary(fitae)
-#  
-#   ### de model
-#   desde <- twin.polygen.design(prt,type="de")
-#   fitde <- survival.twostage(margph,data=prt,theta=1,                            clusters=prt$id,var.link=0,model="clayton.oakes",
-#  numDeriv=1,random.design=desde$des.rv,theta.des=desde$pardes)
-#  
+#  ### ace model with positive random effects variances
+#  # fitacee <- survival.twostage(margph,data=prt,theta=1,
+#  #      clusters=prt$id,var.link=1,model="clayton.oakes",
+#  #      numDeriv=1,random.design=desace$des.rv,theta.des=desace$pardes)
+#  #summary(fitacee)
+# 
+#  ### ae model
+#  #desae <- twin.polygen.design(prt,type="ae")
+#  #fitae <- survival.twostage(margph,data=prt,theta=1,
+#  #      clusters=prt$id,var.link=0,model="clayton.oakes",
+#  #      numDeriv=1,random.design=desae$des.rv,theta.des=desae$pardes)
+#  #summary(fitae)
+# 
+#  ### de model
+#  desde <- twin.polygen.design(prt,type="de")
+#  fitde <- survival.twostage(margph,data=prt,theta=1,                            clusters=prt$id,var.link=0,model="clayton.oakes",
+# numDeriv=1,random.design=desde$des.rv,theta.des=desde$pardes)
+# 
 
 ## -----------------------------------------------------------------------------
 summary(fitde)
@@ -189,8 +189,8 @@ p11dz <- p11$model$"DZ"
  summary(cdz)
  summary(cmz)
 
- timereg::Cpred(cmz$casewise,80)
- timereg::Cpred(cdz$casewise,80)
+ cpred(cmz$casewise,c(70,80))
+ cpred(cdz$casewise,c(70,80))
 
 ## ----label=concordance3-------------------------------------------------------
 
@@ -209,38 +209,38 @@ p11dz <- p11$model$"DZ"
 
 
 ## ----label=concordance4, eval=fullVignette------------------------------------
-#   ### new version of Casewise for specific time-point based on binreg
-#   dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
-#   newdata <- data.frame(zyg=c("DZ","MZ"),id=1)
-#  
-#   ## concordance
-#   bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id),dd,time=80,cause=1,cens.model=~strata(zyg))
-#   pconc <- predict(bcif1,newdata)
-#  
-#   ## marginal estimates
-#   mbcif1 <- binreg(Event(time,status)~cluster(id),prt,time=80,cause=2)
-#   mc <- predict(mbcif1,newdata)
-#  
-#   ### casewise with improved se's from log-scale
-#   cse <- binregCasewise(bcif1,mbcif1)
+#  ### new version of Casewise for specific time-point based on binreg
+#  dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
+#  newdata <- data.frame(zyg=c("DZ","MZ"),id=1)
+# 
+#  ## concordance
+#  bcif1 <- binreg(Event(time,status)~-1+factor(zyg)+cluster(id),dd,time=80,cause=1,cens.model=~strata(zyg))
+#  pconc <- predict(bcif1,newdata)
+# 
+#  ## marginal estimates
+#  mbcif1 <- binreg(Event(time,status)~cluster(id),prt,time=80,cause=2)
+#  mc <- predict(mbcif1,newdata)
+# 
+#  ### casewise with improved se's from log-scale
+#  cse <- binregCasewise(bcif1,mbcif1)
 
 ## -----------------------------------------------------------------------------
  cse 
 
 ## ----label=semiparconc, eval=fullVignette-------------------------------------
-#   ### semi-parametric modelling of concordance
-#   dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
-#   regconc <- cifreg(Event(time,status)~country*zyg,data=dd,prop=NULL)
-#   regconc
-#   ### interaction test
-#   wald.test(regconc,coef.null=5:7)
-#  
-#   regconc <- cifreg(Event(time,status)~country+zyg,data=dd,prop=NULL)
-#   regconc
-#  
-#   ## logistic link
-#   logitregconc <- cifreg(Event(time,status)~country+zyg,data=dd)
-#   slr <- summary(logitregconc)
+#  ### semi-parametric modelling of concordance
+#  dd <- bicompriskData(Event(time,status)~country+strata(zyg)+id(id),data=prt,cause=c(2,2))
+#  regconc <- cifreg(Event(time,status)~country*zyg,data=dd,prop=NULL)
+#  regconc
+#  ### interaction test
+#  wald.test(regconc,coef.null=5:7)
+# 
+#  regconc <- cifreg(Event(time,status)~country+zyg,data=dd,prop=NULL)
+#  regconc
+# 
+#  ## logistic link
+#  logitregconc <- cifreg(Event(time,status)~country+zyg,data=dd)
+#  slr <- summary(logitregconc)
 
 ## -----------------------------------------------------------------------------
 slr
@@ -248,24 +248,29 @@ slr
 ### publish(round(slr$exp.coef[,-c(2,5)],2),latex=TRUE,digits=2)
 
 ## ----label=additive_gamma, eval=fullVignette----------------------------------
-#    times <- seq(50,90,length.out=5)
-#    cif1 <- timereg::comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
-#  		   cause=2,times=times,max.clust=NULL)
-#  
-#    mm <- model.matrix(~-1+factor(zyg),prt)
-#    out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
-#  		  theta.des=mm,same.cens=TRUE,step=0.5)
-#    summary(out1)
-#    round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)
-#  
-#    desace <- twin.polygen.design(prt,type="ace")
-#  
-#    outacem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
-#    	 same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
-#           step=0.5,theta.des=desace$pardes,random.design=desace$des.rv)
-#    ##outacem$score
+#  timereg <- 0
+# if (timereg==1) {
+#   times <- seq(50,90,length.out=5)
+#   cif1 <- timereg::comp.risk(Event(time,status)~-1+factor(country)+cluster(id),prt,
+# 		   cause=2,times=times,max.clust=NULL)
+# 
+#   mm <- model.matrix(~-1+factor(zyg),prt)
+#   out1<-random.cif(cif1,data=prt,cause1=2,cause2=2,theta=1,
+# 		  theta.des=mm,same.cens=TRUE,step=0.5)
+#   summary(out1)
+#   round(estimate(coef=out1$theta,vcov=out1$var.theta)$coefmat[,c(1,3:4)],2)
+# 
+#   desace <- twin.polygen.design(prt,type="ace")
+# 
+#   outacem <- Grandom.cif(cif1,data=prt,cause1=2,cause2=2,
+#   	 same.cens=TRUE,theta=c(0.45,0.15),var.link=0,
+#          step=0.5,theta.des=desace$pardes,random.design=desace$des.rv)
+#   ##outacem$score
+# }
 
 ## -----------------------------------------------------------------------------
+timereg <- 0
+if (timereg==1) {
   summary(outacem)
 
  ###  variances
@@ -306,7 +311,7 @@ slr
  outaem$score
  summary(outaem)
  estimate(coef=outaem$theta,vcov=outaem$var.theta,f=function(p) p/sum(p)^2)
-
+}
 
 ## ----label=probit1------------------------------------------------------------
 rm(prt)
@@ -319,15 +324,15 @@ tau <- 95
 tt <- seq(70, tau, length.out=5) ## Time points to evaluate model in
 
 ## ----b0, eval=fullVignette----------------------------------------------------
-#  b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
-#                cens.formula=Surv(time,status==0)~zyg, breaks=tau)
+# b0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="cor",
+#               cens.formula=Surv(time,status==0)~zyg, breaks=tau)
 
 ## -----------------------------------------------------------------------------
 summary(b0)
 
 ## ----label=liability_ace1, eval=fullVignette----------------------------------
-#  b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
-#                cens.formula=Surv(time,status==0)~zyg, breaks=tau)
+# b1 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
+#               cens.formula=Surv(time,status==0)~zyg, breaks=tau)
 
 ## -----------------------------------------------------------------------------
 summary(b1)
@@ -336,19 +341,19 @@ summary(b1)
 AIC(b0, b1)
 
 ## ----label=liability_ace_country, eval=fullVignette---------------------------
-#  b2 <- bptwin.time(cancer ~ country, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
-#                cens.formula=Surv(time,status==0)~zyg+country, breaks=95)
+# b2 <- bptwin.time(cancer ~ country, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
+#               cens.formula=Surv(time,status==0)~zyg+country, breaks=95)
 
 ## -----------------------------------------------------------------------------
 summary(b2)
 
 ## ----label=bptime1, eval=fullVignette-----------------------------------------
-#  bt0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
-#                cens.formula=Surv(time,status==0)~zyg,
-#                summary.function=function(x) x, breaks=tt)
-#  h2 <- Reduce(rbind, lapply(bt0$coef, function(x) x$heritability))[,c(1,3,4),drop=FALSE]
-#  concMZ <- Reduce(rbind, lapply(bt0$coef, function(x) x$probMZ["Concordance",,drop=TRUE]))
-#  
+# bt0 <- bptwin.time(cancer ~ 1, data=prt0, id="id", zyg="zyg", DZ="DZ", type="ace",
+#               cens.formula=Surv(time,status==0)~zyg,
+#               summary.function=function(x) x, breaks=tt)
+# h2 <- Reduce(rbind, lapply(bt0$coef, function(x) x$heritability))[,c(1,3,4),drop=FALSE]
+# concMZ <- Reduce(rbind, lapply(bt0$coef, function(x) x$probMZ["Concordance",,drop=TRUE]))
+# 
 
 ## -----------------------------------------------------------------------------
 par(mfrow=c(1,2))
@@ -378,31 +383,31 @@ plot(a.mz, ylim=c(0,.07), col=cols[1], lty=ltys[1], legend=FALSE, add=TRUE)
 plot(a.dz, col=cols[2], lty=ltys[2], add=TRUE)
 
 ## ----label=biprobittime2, eval=fullVignette-----------------------------------
-#  a.mz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="MZ"),
-#                                 cens.formula = Surv(time,status==0)~country, pairs.only=TRUE,
-#                                  breaks=tt)
-#  system.time(a.dz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="DZ"),
-#                                 cens.formula = Event(time,status==0)~country, pairs.only=TRUE,
-#                                 breaks=tt))
-#  
-#  s_mz_country <- summary(a.mz_country)
-#  s_dz_country <- summary(a.dz_country)
+# a.mz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="MZ"),
+#                                cens.formula = Surv(time,status==0)~country, pairs.only=TRUE,
+#                                 breaks=tt)
+# system.time(a.dz_country <- biprobit.time(cancer~country, id="id", data=subset(prt0, zyg=="DZ"),
+#                                cens.formula = Event(time,status==0)~country, pairs.only=TRUE,
+#                                breaks=tt))
+# 
+# s_mz_country <- summary(a.mz_country)
+# s_dz_country <- summary(a.dz_country)
 
 ## -----------------------------------------------------------------------------
 s_mz_country
 s_dz_country
 
 ## ----label=liability_ace_time1, eval=fullVignette-----------------------------
-#  ## ACE model (time-varying) with and without adjustment for country
-#  a1 <- bptwin.time(cancer~1, id="id", data=prt0, type="ace",
-#                                zyg="zyg", DZ="DZ",
-#                                cens.formula=Surv(time,status==0)~zyg,
-#                                breaks=tt)
-#  
-#  #a2 <- bptwin.time(cancer~country, id="id", data=prt0, #type="ace",
-#  #                              zyg="zyg", DZ="DZ",
-#  #                              #cens.formula=Surv(time,status==0)~country+zyg,
-#  #                              breaks=tt)
+# ## ACE model (time-varying) with and without adjustment for country
+# a1 <- bptwin.time(cancer~1, id="id", data=prt0, type="ace",
+#                               zyg="zyg", DZ="DZ",
+#                               cens.formula=Surv(time,status==0)~zyg,
+#                               breaks=tt)
+# 
+# #a2 <- bptwin.time(cancer~country, id="id", data=prt0, #type="ace",
+# #                              zyg="zyg", DZ="DZ",
+# #                              #cens.formula=Surv(time,status==0)~country+zyg,
+# #                              breaks=tt)
 
 ## -----------------------------------------------------------------------------
 plot(a.mz, which=c(6), xlab="Age", ylab="Correlation", ylim=c(0,1), col=cols[1], lty=ltys[1], legend=NULL, alpha=.1)
@@ -425,32 +430,32 @@ plot(a1, which=c(1), xlab="Age", ylim=c(0,1), col="black", lty=1, ylab="Heritabi
 sessionInfo()
 
 ## ----saveobj, results="hide", echo=FALSE, eval=fullVignette-------------------
-#  ## To save time building the vignettes on CRAN, we cache time consuming computations
-#  
-#  rms <- c('id','theta.iid','theta.des','marginal.trunc',
-#    'loglikeiid','marginal.surv','theta.iid.naive',
-#    'antclust','secluster','cluster.call','trunclikeiid',
-#    'logl.iid','score.iid','clusters')
-#  tmp <- lapply(as.list(paste0("fitco", 1:4)),
-#                function(x) saveobj(x, rms))
-#  
-#  rms <- c('random.design','marginal.surv','marginal.trunc','theta.iid','score.iid','loglikeiid','trunclikeiid','antclust','cluster.call','secluster','clusters')
-#  saveobj("fitace", rms)
-#  saveobj("fitde", rms)
-#  
-#  saveobj("cse", NULL)
-#  saveobj("slr", NULL)
-#  
-#  rms <- c("theta.iid", "Clusters", "p11")
-#  saveobj("outacem", rms)
-#  
-#  rms <- c("model.frame", "score", "id", "logLik")
-#  saveobj("b0", rms)
-#  saveobj("b1", rms)
-#  saveobj("b2", rms)
-#  saveobj("a1", rms)
-#  saveobj("h2", NULL)
-#  saveobj("concMZ", NULL)
-#  saveobj("s_mz_country", NULL)
-#  saveobj("s_dz_country", NULL)
+# ## To save time building the vignettes on CRAN, we cache time consuming computations
+# 
+# rms <- c('id','theta.iid','theta.des','marginal.trunc',
+#   'loglikeiid','marginal.surv','theta.iid.naive',
+#   'antclust','secluster','cluster.call','trunclikeiid',
+#   'logl.iid','score.iid','clusters')
+# tmp <- lapply(as.list(paste0("fitco", 1:4)),
+#               function(x) saveobj(x, rms))
+# 
+# rms <- c('random.design','marginal.surv','marginal.trunc','theta.iid','score.iid','loglikeiid','trunclikeiid','antclust','cluster.call','secluster','clusters')
+# saveobj("fitace", rms)
+# saveobj("fitde", rms)
+# 
+# saveobj("cse", NULL)
+# saveobj("slr", NULL)
+# 
+# rms <- c("theta.iid", "Clusters", "p11")
+# saveobj("outacem", rms)
+# 
+# rms <- c("model.frame", "score", "id", "logLik")
+# saveobj("b0", rms)
+# saveobj("b1", rms)
+# saveobj("b2", rms)
+# saveobj("a1", rms)
+# saveobj("h2", NULL)
+# saveobj("concMZ", NULL)
+# saveobj("s_mz_country", NULL)
+# saveobj("s_dz_country", NULL)
 
