@@ -397,7 +397,7 @@ RcppExport SEXP BhatAddGamCC(SEXP itwostage,SEXP idBaalen,SEXP icause,
         // computes weights based on additive gamma model 
 	// cumulative hazards for cases Xcase^T B(T_case)
         vec cumhazcase=trans(Bcaseit.row(k)); 
-        if ((recursive==0) & (k>0)) cumhaz=xjump.slice(k)*trans(Bit.row(k)); 
+        if ((recursive==0) && (k>0)) cumhaz=xjump.slice(k)*trans(Bit.row(k));
         cumhazt=xjump.slice(k)*trans(Bit.row(k)); 
 
         //int lnrv= nrvs(k)-1; // number of random effects for this cluster 	
@@ -578,52 +578,3 @@ try {
 //  }
 //  return R_NilValue; // -Wall
 //} // }}}
-
-//arma::uvec FastApproxC(vec time, vec newtime,int equal,int type 
-//		// (0: nearest, 1: right, 2: left)
-//	        ) {/*{{{*/
-//  int Type = Rcpp::as<int>(type);
-//  NumericVector NewTime(newtime);
-//  NumericVector Sorted(time);
-//  // IntegerVector Order;
-//  // NumericVector Sorted = Time;
-//  // std::sort(Sorted.begin(), Sorted.end());
-//  // //    .sort();
-//  // IntegerVector Order = match(Sorted, Time);
-//  // return(Rcpp::wrap(Time));  
-//  int Equal  = Rcpp::as<int>(equal);
-////  bool Equal = Rcpp::as<bool>(equal);
-//  vector<int> eq(NewTime.size());
-//  vector<int> idx(NewTime.size());
-////  IntegerVector idx(NewTime.size(),0);
-//
-//  double vmax = Sorted[Sorted.size()-1];
-//  NumericVector::iterator it;  
-//  double upper=0.0; int pos=0;
-//  for (int i=0; i<NewTime.size(); i++) {    
-//    eq[i] = 0;
-//    if (NewTime[i]>vmax) {
-//      pos = Sorted.size()-1;
-//    } else {
-//      it = lower_bound(Sorted.begin(), Sorted.end(), NewTime[i]);
-//      upper = *it;
-//      if (it == Sorted.begin()) { 
-//	pos = 0; 
-//	if ((Equal==1) && (NewTime[i]==upper)) { eq[i] = 1; }
-//      }
-//      // else if (int(it-Sorted.end())==0) {
-//      // 	pos = Sorted.size()-1;
-//      // }
-//      else {
-//	pos = int(it-Sorted.begin());
-//	if (Type==0 && fabs(NewTime[i]-Sorted[pos-1])<fabs(NewTime[i]-Sorted[pos])) pos -= 1;
-//	if ((Equal==1) && (NewTime[i]==upper)) { eq[i] = pos+1; }
-//      }
-//    }
-//    if (Type==2 && NewTime[i]<upper) pos--;
-//    idx[i] = pos+1;
-//  }
-//  return(idx); 
-//
-//}/*}}}*/
-//
